//console command that swaps the functions of spin and custom 1 for controllers
COM_AddCommand("crash_consolemode", function(player, var)
	if var == "off" or var == "0"
		player.crash_consolemode = false
		CONS_Printf(player, "Console Mode is disabled!")
	elseif var == "on" or var == "1"
		player.crash_consolemode = true
		CONS_Printf(player, "Console Mode is enabled!")
	else
		CONS_Printf(player, "Console Mode swaps the actions of the spin and " ..
				"custom 1 buttons, useful for controller players. Use on/off" ..
				" or 1/0 to enable/disable it.")
	end
end)

local function MapReloader()
	if gamemap != nil and (G_GametypeUsesLives() and not G_CompetitionGametype())
		G_SetCustomExitVars(gamemap, 2)
		G_ExitLevel()
	end
end

//console command that enables or disables the box tally and boxes
COM_AddCommand("crash_boxtallymode", function(player, var)
	if var == "off" or var == "0"
		CrashAddon.boxtallymode = false
		print("Box Tally mode has been disabled!")
		MapReloader()
	elseif var == "on" or var == "1"
		CrashAddon.boxtallymode = true
		print("Box Tally mode has been enabled!")
		MapReloader()
	else
		CONS_Printf(player, "Box Tally Mode replaces most monitors with " ..
				"boxes and gives a tally to keep track of how many have been" ..
				" broken. Only works in Singleplayer and Co-op, and requires" ..
				" admin permissions to use. Use on/off or 1/0 to enable/" ..
				"disable it. WARNING: Current map will be reloaded!")
	end
end, COM_ADMIN)

//console command that enables/disables the box pointer for all players to use
COM_AddCommand("crash_globalpointer", function(player, var)
	if var == "off" or var == "0"
		CrashAddon.globalpointer = false
		print("Box Pointer has been disabled globally!")
	elseif var == "on" or var == "1"
		if CrashAddon.boxtallymode
			CrashAddon.globalpointer = true
			print("Box Pointer has been enabled globally!")
		else
			CONS_Printf(player, "Box Tally Mode must be enabled for this to" ..
					" work!")
		end
	else
		CONS_Printf(player, "Enables or disables the use of Box Pointers. " ..
				"Requires admin permissions and Box Tally Mode to be " ..
				"enabled. Use on/off or 1/0 to enable/disable it.")
	end
end, COM_ADMIN)

//console command to enable the box pointer client-side
COM_AddCommand("crash_boxpointer", function(player, var)
	if var == "off" or var == "0"
		player.crash_boxpointer = false
		CONS_Printf(player, "Box Pointer is disabled!")
	elseif var == "on" or var == "1"
		if CrashAddon.boxtallymode
			if CrashAddon.globalpointer
				player.crash_boxpointer = true
				CONS_Printf(player, "Box Pointer is enabled!")
			else
				CONS_Printf(player, "No can do! Box Pointer is disabled on " ..
						"this server!")
			end
		else
			CONS_Printf(player, "Box Tally Mode must be enabled for this to " ..
					"work!")
		end
	else
		CONS_Printf(player, "Box Pointer spawns a compass that points " ..
				"towards the closest box. Only works if Box Tally Mode is " ..
				"on and if admins allow Box Pointers globally. Use on/off " ..
				"or 1/0 to enable/disable it.")
	end
end)

//command to toggle the singleplayer box totals for multiplayer
COM_AddCommand("crash_easymultiplayer", function(player, var)
	if var == "off" or var == "0"
		CrashAddon.easymultiplayer = false
		print("Easy mode for multiplayer has been disabled!")
		MapReloader()
	elseif var == "on" or var == "1"
		if CrashAddon.boxtallymode
			CrashAddon.easymultiplayer = true
			print("Easy mode for multiplayer has been enabled!")
			MapReloader()
		else
			CONS_Printf(player, "Box Tally Mode must be enabled for this to " ..
					"work!")
		end
	else
		CONS_Printf(player, "Easy mode for multiplayer enables the " ..
				"singleplayer box totals for multiplayer. For small lobbies " ..
				"or gigantic maps. Requires admin permissions and Box Tally " ..
				"Mode to be enabled for this to work. Use on/off or 1/0 to " ..
				"enable/disable it. WARNING: Current map will be reloaded!")
	end
end, COM_ADMIN)

//a function to ensure box-tally-related scripts only run when conditions are met
CrashAddon.BoxTallyMode = function()
	return CrashAddon.boxtallymode and (G_GametypeUsesLives() and not 
			G_CompetitionGametype())
end

//a function to make skinchecking easier
CrashAddon.CrashSkinCheck = function(player)
	if player.mo == nil
		return false
	end
	if player.mo.skin != "crashbandi"
		return false
	end
	return true
end

//this function just makes applying z momentum easier to manage
CrashAddon.JumpManager = function(mo, hight, relative)
	//this nerfs the jumps while underwater or in space
	if mo.eflags & MFE_UNDERWATER or mo.eflags & MFE_GOOWATER or 
			P_InSpaceSector(mo)
		hight = $ - ($ / 3)
	end
	P_SetObjectMomZ(mo, hight, relative)
end

//this makes setting states easier
CrashAddon.StateSetter = function(mo, sstate, powflags)
	if mo.state != sstate
		if powflags != nil
			mo.player.powers[pw_strong] = powflags
		end
		mo.state = sstate
	end
end

//speed calculation because player.speed not be accurate
CrashAddon.SpeedCalc = function(player)
	local horispeed = FixedHypot(player.mo.momx - player.cmomx, player.mo.momy -
			player.cmomy)
	return FixedHypot(horispeed, player.mo.momz)
end

//a function that returns the button to check based on if console mode is on/off
CrashAddon.ActionButton = function(player)
	if player.crash_consolemode
		return BT_CUSTOM1
	else
		return BT_SPIN
	end
end

//this makes checking for when moves should be disabled easier
local StateChecker = {
	[S_PLAY_PAIN] = true,
	[S_PLAY_GASP] = true,
	[S_PLAY_SUPER_TRANS1] = true,
	[S_PLAY_SUPER_TRANS2] = true,
	[S_PLAY_SUPER_TRANS3] = true,
	[S_PLAY_SUPER_TRANS4] = true,
	[S_PLAY_SUPER_TRANS5] = true,
	[S_PLAY_SUPER_TRANS6] = true
}
CrashAddon.FailVibeCheck = function(player)
	return P_PlayerInPain(player) or player.playerstate == PST_DEAD or
			player.powers[pw_carry] != CR_NONE or StateChecker[player.mo.state]
			or player.crashban.transtimer >= 0
end

//made this a function because i realized i use this check a lot
CrashAddon.CootOnGround = function(mo)
	return P_IsObjectOnGround(mo) or mo.eflags & MFE_JUSTHITFLOOR
end

//function for checking if crash is falling by checking momentum
CrashAddon.IsFalling = function(mo)
	if mo.eflags & MFE_VERTICALFLIP
		return mo.momz >= 0
	else
		return mo.momz <= 0
	end
end

//added in to check if the player is in nights mode
CrashAddon.IsNiGHTS = function(player)
	return player.powers[pw_carry] == CR_NIGHTSMODE or player.powers[pw_carry] == 
			CR_NIGHTSFALL
end

//making this into a standalone function for easy setup
CrashAddon.DamageRadius = function(mo, radius)
	searchBlockmap("objects", function(mo, found)
		if found != nil and found.valid and found.health and P_CheckSight(mo, 
				found) and found.z <= mo.z + radius and found.z >= mo.z - radius
			P_DamageMobj(found, mo, mo, 1)
		end
	end, mo, mo.x - radius, mo.x + radius, mo.y - radius, mo.y + radius)
end

//function to make spawning objects relative to other objects easier
CrashAddon.ObjectSpawner = function(mo, tipe, x, y, z)
	x = $ or 0
	y = $ or 0
	z = $ or 0
	local spawn = P_SpawnMobjFromMobj(mo, 0, 0, 0, tipe)
	spawn.scale = mo.scale
	spawn.eflags = mo.eflags
	if x != 0 or y != 0 or z != 0
		P_MoveOrigin(spawn, mo.x + x, mo.y + y, mo.z + z)
	end
	return spawn
end

//having to make a copy of P_SuperReady because it only works if you have SF_SUPER GRAAHHHHHHHH
CrashAddon.CanSupper = function(player)
	return All7Emeralds(emeralds) and player.rings >= 50 and player.pflags & 
			PF_JUMPED and not player.powers[pw_shield] and not 
			player.powers[pw_super]
end

//whyyyyy is super so finickyyyyyyyy
CrashAddon.SuperSprites = {
	[S_PLAY_SUPER_TRANS1] = true,
	[S_PLAY_SUPER_TRANS2] = true,
	[S_PLAY_SUPER_TRANS3] = true,
	[S_PLAY_SUPER_TRANS4] = true,
	[S_PLAY_SUPER_TRANS5] = true,
	[S_PLAY_SUPER_TRANS6] = true
}

//makes checking for what camera is tied to the player easier
CrashAddon.PlayerCamera = function(player)
	local cam
	if displayplayer == player
		cam = camera
	elseif secondarydisplayplayer == player
		cam = camera2
	end
	return cam
end