local IsNiGHTS = CrashAddon.IsNiGHTS
local ObjectSpawner = CrashAddon.ObjectSpawner
local SuperSprites = CrashAddon.SuperSprites
local PlayerCamera = CrashAddon.PlayerCamera

local SuperColors = {
	[SKINCOLOR_SUPERORANGE1] = SKINCOLOR_SUPERGOLD1,
	[SKINCOLOR_SUPERORANGE2] = SKINCOLOR_SUPERGOLD2,
	[SKINCOLOR_SUPERORANGE3] = SKINCOLOR_SUPERGOLD3,
	[SKINCOLOR_SUPERORANGE4] = SKINCOLOR_SUPERGOLD4,
	[SKINCOLOR_SUPERORANGE5] = SKINCOLOR_SUPERGOLD5
}

//this handles Aku Aku's behavior
local function AkuAkuThinker(mo)
	//failsafe
	if not mo.target or not mo.target.valid or not mo.target.player or 
			IsNiGHTS(mo.target.player) or mo.target.player.playerstate !=
			PST_LIVE
		P_RemoveMobj(mo)
		return
	end
	local player = mo.target.player
	local target = mo.target
	local akulevel = player.crashban.akulevel
	if akulevel == nil or akulevel <= 0
		P_RemoveMobj(mo)
		return
	end
	//this determines sound cues
	if mo.currentlevel == nil
		mo.currentlevel = akulevel
	end
	if mo.currentlevel != akulevel
		if mo.currentlevel < akulevel and akulevel != 4
			S_StartSound(target, sfx_akuspn, player)
		end
		mo.currentlevel = akulevel
	end
	//this makes aku aku semi-transparent if the player is in first-person
	local cam = PlayerCamera(player)
	if cam != nil and not cam.chase
		mo.frame = $ | FF_TRANS80
	else
		mo.frame = $ & ~FF_TRANS80
	end
	mo.scale = target.scale
	mo.shadowscale = mo.scale
	mo.eflags = target.eflags
	if SuperSprites[target.state]
		mo.flags2 = $ | MF2_DONTDRAW
	else
		mo.flags2 = $ & ~MF2_DONTDRAW
	end
	local isSuper = player.powers[pw_super] or SuperSprites[target.state] or 
			player.crashban.supertrans
	//this changes Aku Aku's color based on aku level
	if akulevel > 1
		if isSuper and akulevel <= 3
			mo.colorized = true
			mo.color = SuperColors[target.color] or SKINCOLOR_GOLDENROD
			mo.frame = $ | FF_FULLBRIGHT
		elseif akulevel <= 3
			mo.colorized = true
			mo.frame = $ & ~FF_FULLBRIGHT
			if leveltime % 3 == 0
				if akulevel > 2 and mo.color == SKINCOLOR_GOLDENROD
					mo.color = SKINCOLOR_SUPERGOLD2
				else
					mo.color = SKINCOLOR_GOLDENROD
				end
			end
		end
	else
		mo.colorized = false
	end
	local invincibility = player.powers[pw_invulnerability]
	//this makes Aku Aku orbit around the player
	if akulevel < 3
		if invincibility or isSuper
			player.crashban.akulevel = 3
		end
		mo.threshold = 40 * mo.scale
		A_UnidusBall(mo)
		mo.angle = R_PointToAngle2(target.x, target.y, mo.x, mo.y)
		//this is all the fancy visual effects
		if akulevel > 1 and leveltime % 7 == 0
			local akusparks = ObjectSpawner(mo, MT_IVSP)
			akusparks.colorized = true
			akusparks.frame = $ | FF_ADD | FF_TRANS70
			akusparks.color = SKINCOLOR_GOLDENROD
		end
	//this gives Crash invincibility and moves Aku Aku once max level is reached
	elseif akulevel == 3
		A_ForceStop(mo, 0)
		if not invincibility and not isSuper
			if not mo.invin
				if not isSuper
					A_Invincibility(mo)
				end
				mo.invin = true
			elseif not isSuper
				mo.invin = false
				player.crashban.akulevel = 2
				S_StartSound(target, sfx_akuded, player)
			end
		elseif not mo.invin
			mo.invin = true
		end
		//this makes aku aku be in front of crash's face
		mo.angle = player.drawangle
		local akux = target.x + P_ReturnThrustX(mo, mo.angle, 23 * mo.scale)
		local akuy = target.y + P_ReturnThrustY(mo, mo.angle, 23 * mo.scale)
		local akuheight
		if mo.eflags & MFE_VERTICALFLIP
			akuheight = target.height / 3
		else
			akuheight = target.height - target.height / 3 - target.height / 10
		end
		P_MoveOrigin(mo, akux, akuy, target.z + akuheight)
	end
end

//hooks
addHook("MobjThinker", AkuAkuThinker, MT_AKUAKU)