local CrashSkinCheck = CrashAddon.CrashSkinCheck
local JumpManager = CrashAddon.JumpManager
local StateSetter = CrashAddon.StateSetter
local ObjectSpawner = CrashAddon.ObjectSpawner

//table for different sound effects to accompany different deaths
local DeathSounds = {
	[DMG_FIRE] = sfx_ashcsh,
	[DMG_ELECTRIC] = sfx_cshxry,
	[DMG_SPIKE] = sfx_s3k77,
	[DMG_NUKE] = sfx_cshlgo,
	[DMG_SPACEDROWN] = sfx_cshfrz,
	[DMG_DROWNED] = sfx_crdrwn,
	[DMG_DEATHPIT] = sfx_cshfal
}

//function to spawn all of crash's bits
local function GibSpawner(mo, fram)
	local gib = ObjectSpawner(mo, MT_CRASHBITS)
	gib.frame = fram
	gib.tracer = mo
	local anglrng = P_RandomRange(180 * -1, 180) * FRACUNIT
	local rndmangl = gib.angle + FixedAngle(anglrng)
	P_InstaThrust(gib, rndmangl, P_RandomRange(1, 5) * gib.scale)
	JumpManager(gib, P_RandomRange(1, 5) * FRACUNIT)
end

//table for all the different explosions for explosive deaths
local Explosives = {
	[MT_TNTBARREL] = true,
	[MT_FBOMB] = true,
	[MT_CYBRAKDEMON_MISSILE] = true,
	[MT_BIGMINE] = true,
	[MT_BLASTEXECUTOR] = true,
	[MT_DRAGONMINE] = true,
	[MT_MINE] = true,
	[MT_PROXIMITYTNT] = true,
	[MT_MINECART] = true,
	[MT_THROWNEXPLOSION] = true,
	[MT_DETON] = true
}

//this sets things up for the custom deaths
local RagdollDeaths = {
	[DMG_ELECTRIC] = true,
	[DMG_SPACEDROWN] = true,
	[DMG_CRUSHED] = true
}

local function DeathAnimTrigger(target, inflictor, source, damagetype)
	if target.skin != "crashbandi"
		return
	end
	
	target.deathtype = damagetype
	//having explosive things change deathtypes for explosion death
	if inflictor and inflictor.valid
		if Explosives[inflictor.type] and target.deathtype != DMG_NUKE
			target.deathtype = DMG_NUKE
		end
	end
	//override drowning in space for the space death
	if P_InSpaceSector(target) and target.deathtype == DMG_DROWNED
		target.deathtype = DMG_SPACEDROWN
	end
	//spawning and killing a balloon for an orange POP-sicle, aye? AYE??
	if target.deathtype == DMG_SPIKE
		target.flags2 = $ | MF2_DONTDRAW
		local poppedbandi = ObjectSpawner(target, MT_BALLOON)
		poppedbandi.color = target.player.skincolor
		P_KillMobj(poppedbandi)
	//spawn a bunch of crash pieces and scatter them for funni reference
	elseif target.deathtype == DMG_NUKE
		target.flags2 = $ | MF2_DONTDRAW
		GibSpawner(target, A)
		GibSpawner(target, A|FF_HORIZONTALFLIP)
		GibSpawner(target, B)
		GibSpawner(target, C)
		GibSpawner(target, D)
	//spawns in crash's "corpse" for deaths that stay on screen
	elseif RagdollDeaths[target.deathtype]
		local corpse = ObjectSpawner(target, MT_CRASHBITS)
		corpse.tracer = target
		corpse.skin = target.skin
		corpse.color = target.player.skincolor
		corpse.deathtype = target.deathtype
		corpse.height = target.height
		corpse.radius = target.radius
		corpse.angle = target.angle
	end
	S_StartSound(target, DeathSounds[target.deathtype] or sfx_none)
end

//actual functions for the different deaths
local CorpseTypes = {
	//redneck's x-ray
	[DMG_ELECTRIC] = function(mo)
		A_ForceStop(mo, 0)
		StateSetter(mo, S_PLAY_DEAD)
		mo.flags = MF_NOCLIP|MF_NOGRAVITY|MF_SCENERY|MF_NOCLIPHEIGHT
		if leveltime % 2 == 0
			if mo.frame & FF_ADD and mo.frame & FF_FULLBRIGHT
				mo.frame = $ & ~FF_ADD & ~FF_FULLBRIGHT
			else
				mo.frame = $ | FF_ADD | FF_FULLBRIGHT
			end
		end
		local radius = mo.radius / mo.scale
		local hight = mo.height / 2 / mo.scale
		local staticx = P_RandomRange(radius * -1, radius) * mo.scale
		local staticy = P_RandomRange(radius * -1, radius) * mo.scale
		local staticz = P_RandomRange(hight * -1, hight) * mo.scale
		local staticfx = ObjectSpawner(mo, MT_WATERZAP, staticx, staticy, 
				mo.height / 2 + staticz)
		staticfx.frame = $ | FF_ADD
		staticfx.colorized = true
		staticfx.color = SKINCOLOR_WHITE
	end,
	//suffocating in space, could be a lot worse
	[DMG_SPACEDROWN] = function(mo)
		if not mo.colorized
			mo.colorized = true
			mo.color = SKINCOLOR_ICY
		end
		A_ForceStop(mo, 0)
		StateSetter(mo, S_PLAY_DEAD)
		mo.flags = MF_NOCLIP|MF_NOGRAVITY|MF_SCENERY|MF_NOCLIPHEIGHT
	end,
	//we be making bandicoot pancakes
	[DMG_CRUSHED] = function(mo)
		A_ForceStop(mo, 0)
		StateSetter(mo, S_PLAY_DEAD)
		mo.flags = MF_NOCLIP|MF_NOGRAVITY|MF_SCENERY|MF_NOCLIPHEIGHT
		mo.frame = $ | FF_FLOORSPRITE
		mo.renderflags = $ | RF_NOSPLATBILLBOARD
	end
}

//simple thinker that kills crash's "corpse" when the player respawns
local function CorpseThinker(mo)
	if mo.tracer == nil or not mo.tracer.valid or mo.tracer.skin != "crashbandi"
		P_RemoveMobj(mo)
		return
	end
	if mo.deathtype and RagdollDeaths[mo.deathtype]
		CorpseTypes[mo.deathtype](mo)
	end
end

local function NoDraw(mo)
	mo.flags2 = $ | MF2_DONTDRAW
	JumpManager(mo, 2 * FRACUNIT / 3)
end

local DeathTypes = {
	//overcooked marsupial
	[DMG_FIRE] = function(mo)
		if not mo.fade
			mo.fade = 1
		end
		if not mo.colorized
			mo.colorized = true
			mo.color = SKINCOLOR_BLACK
		end
		JumpManager(mo, 2 * FRACUNIT / 3)
		if leveltime % 7 == 0
			if mo.fade <= 9
				mo.frame = $ | (mo.fade << 16)
				mo.fade = $ + 1
			end
		end
		if mo.fade <= 9
			local radius = mo.radius / mo.scale
			local ashx = P_RandomRange(radius * -1, radius) * mo.scale
			local ashy = P_RandomRange(radius * -1, radius) * mo.scale
			local ashfx = ObjectSpawner(mo, MT_DUST, ashx, ashy)
			ashfx.colorized = true
			ashfx.color = SKINCOLOR_BLACK
		end
	end,
	[DMG_ELECTRIC] = NoDraw,
	[DMG_SPIKE] = NoDraw,
	[DMG_NUKE] = NoDraw,
	[DMG_SPACEDROWN] = NoDraw,
	//falling, forgot the cartoon sign
	[DMG_DEATHPIT] = function(mo)
		if mo.momz >= 0
			JumpManager(mo, -50 * FRACUNIT)
		end
		StateSetter(mo, S_PLAY_FALL)
	end,
	[DMG_CRUSHED] = NoDraw
}

local ValidDeaths = {
	[DMG_FIRE] = true,
	[DMG_ELECTRIC] = true,
	[DMG_SPIKE] = true,
	[DMG_NUKE] = true,
	[DMG_SPACEDROWN] = true,
	[DMG_DEATHPIT] = true,
	[DMG_CRUSHED] = true
}

CrashAddon.DeathAnimations = function(player)
	if not CrashSkinCheck(player)
		return
	end
	
	if player.playerstate == PST_DEAD and player.mo.deathtype and 
			ValidDeaths[player.mo.deathtype]
		player.mo.flags = $ & ~MF_NOGRAVITY
		DeathTypes[player.mo.deathtype](player.mo)
	end
end

//hooks
addHook("MobjDeath", DeathAnimTrigger, MT_PLAYER)
addHook("MobjThinker", CorpseThinker, MT_CRASHBITS)