local CrashSkinCheck = CrashAddon.CrashSkinCheck
local ActionButton = CrashAddon.ActionButton
local JumpManager = CrashAddon.JumpManager
local FailVibeCheck = CrashAddon.FailVibeCheck
local CootOnGround = CrashAddon.CootOnGround
local IsFalling = CrashAddon.IsFalling
local DamageRadius = CrashAddon.DamageRadius
local CanSupper = CrashAddon.CanSupper

//has its own statesetter function due to the strong power flags and same state
local function StateSetter(player)
	CrashAddon.StateSetter(player.mo, S_PLAY_MELEE, STR_ANIM|STR_FLOOR|
			STR_SPRING|STR_HEAVY|STR_STOMP)
end

//local variables to determine belly flop radius
local flop_radius = 75
local super_flop_radius = 150

CrashAddon.BodySlam = function(player)
	if not CrashSkinCheck(player)
		return
	end
	
	local crashban = player.crashban
	//this resets everything and prevents moves if crash fails the vibe check
	if FailVibeCheck(player) or crashban.ropedrop or (CanSupper(player) and 
			not player.crash_consolemode)
		crashban.bodyslam = false
		crashban.hover = -1
		crashban.faceplant = 0
		player.height = skins[player.mo.skin].height
		return
	end
	
	//mostly ported from Emmette, but with some tweaks to optimize it
	//this triggers the body slam if the right conditions are met
	if (player.pflags & PF_JUMPED or player.mo.state == S_PLAY_FALL) and not 
			(CootOnGround(player.mo) or player.mo.state == S_PLAY_SPINDASH) and 
			player.cmd.buttons & ActionButton(player) and not 
			(player.cmd.buttons & BT_JUMP or player.mo.eflags & MFE_GOOWATER)
		if not crashban.bodyslam and not crashban.actionpress
			A_ForceStop(player.mo, 0)
			S_StartSound(player.mo, sfx_cshflp)
			JumpManager(player.mo, 7 * FRACUNIT)
			crashban.hover = TICRATE
			crashban.bodyslam = true
			crashban.actionpress = true
		end
	end
	//this handles what happens during the slam
	if crashban.bodyslam
		player.pflags = $ | PF_FULLSTASIS
		player.height = player.spinheight
		//this makes crash hover in the air briefly
		if crashban.hover > 0
			if player.mo.momz == 0
				player.mo.momz = 0
			end
			StateSetter(player)
			crashban.hover = max($ - 4, 0)
		//this makes crash fall once he's done hovering
		elseif crashban.hover == 0
			JumpManager(player.mo, -40 * FRACUNIT)
			crashban.hover = $ - 1
		else
			if player.mo.eflags & MFE_SPRUNG or player.mo.eflags & MFE_GOOWATER
					or not IsFalling(player.mo)
				crashban.bodyslam = false
			//this handles what happens while crash is falling
			elseif not CootOnGround(player.mo)
				StateSetter(player)
			//this spawns the attack radius for actually impacting the ground
			else
				local radius
				if player.powers[pw_super]
					radius = super_flop_radius
				else
					radius = flop_radius
				end
				S_StartSound(player.mo, sfx_pstop)
				P_Earthquake(player.mo, player.mo, (radius + 25) * 
						player.mo.scale)
				DamageRadius(player.mo, radius * player.mo.scale)
				player.mo.state = S_PLAY_MELEE_LANDING
				crashban.faceplant = TICRATE
				crashban.bodyslam = false
			end
		end
	end
	//this handles what happens after crash has slammed down
	if crashban.faceplant > 0
		player.pflags = $ | PF_FULLSTASIS
		player.height = player.spinheight
		crashban.faceplant = $ - 3
	elseif player.mo.state == S_PLAY_MELEE_LANDING
		player.mo.state = S_PLAY_STND
		player.height = skins[player.mo.skin].height
	end
end

addHook("PreThinkFrame", function()
	for player in players.iterate do
		if not CrashSkinCheck(player)
			continue
		end
		
		local crashban = player.crashban
		if player.cmd.buttons & BT_SPIN and player.powers[pw_carry] ==
				CR_ROPEHANG
			crashban.ropedrop = true
		elseif (FailVibeCheck(player) or CootOnGround(player.mo))
				and crashban.ropedrop
			crashban.ropedrop = false
		end
	end
end)