local CrashSkinCheck = CrashAddon.CrashSkinCheck
local StateSetter = CrashAddon.StateSetter
local SpeedCalc = CrashAddon.SpeedCalc
local ActionButton = CrashAddon.ActionButton
local FailVibeCheck = CrashAddon.FailVibeCheck
local CootOnGround = CrashAddon.CootOnGround
local IsFalling = CrashAddon.IsFalling
local CanSupper = CrashAddon.CanSupper

//local variables to handle slide and crawl stuff
local add_slide_speed = 10
local super_slide_speed = 30
local slide_timer = TICRATE / 2
local super_slide_timer = TICRATE - TICRATE / 4
local crawl_speed = 5 * FRACUNIT
local super_crawl_speed = 10 * FRACUNIT

CrashAddon.CrawlAndSlide = function(player)
	if not CrashSkinCheck(player)
		return
	end
	
	local crashban = player.crashban
	//this resets everything and prevents moves if crash fails the vibe check
	if FailVibeCheck(player) or (CanSupper(player) and not 
			player.crash_consolemode)
		player.normalspeed = skins[player.mo.skin].normalspeed
		player.height = skins[player.mo.skin].height
		crashban.slidetimer = 0
		return
	end
	
	//this resets the height and speed changes made by crawling and sliding
	if not (player.cmd.buttons & ActionButton(player)) and crashban.actionpress
		crashban.actionpress = false
	end
	if crashban.slidetimer <= 0 and not crashban.bodyslam and
			crashban.faceplant <= 0 and not crashban.actionpress and
			(player.height < skins[player.mo.skin].height or
			player.normalspeed < skins[player.mo.skin].normalspeed)
		player.height = skins[player.mo.skin].height
		if player.normalspeed < skins[player.mo.skin].normalspeed
			player.normalspeed = skins[player.mo.skin].normalspeed
		end
	end
	//this handles what happens when crash hits a spring while crawling/sliding
	if (player.cmd.buttons & ActionButton(player) or crashban.slidetimer > 0) 
			and player.mo.eflags & MFE_SPRUNG
		if CootOnGround(player.mo)
			player.mo.state = S_PLAY_ROLL
			player.pflags = $ | PF_SPINNING
		else
			player.mo.state = S_PLAY_SPRING
		end
		crashban.slidetimer = 0
	end
	//this makes Crash crawl or start sliding
	local crawlspeed
	if player.powers[pw_super]
		crawlspeed = super_crawl_speed
	else
		crawlspeed = crawl_speed
	end
	if player.cmd.buttons & ActionButton(player) and CootOnGround(player.mo)
			and crashban.spintimer <= 0 and not (player.pflags & PF_SPINNING)
		player.height = player.spinheight
		if not crashban.actionpress
			if SpeedCalc(player) > 1 * player.mo.scale and 
					crashban.slidetimer <= 0
				//sets up slide speed and timer
				local speed
				local timer
				if player.powers[pw_super]
					speed = super_slide_speed
					timer = super_slide_timer
				else
					speed = add_slide_speed
					timer = slide_timer
				end
				local slidespeed
				if SpeedCalc(player) > 30 * player.mo.scale
					slidespeed = SpeedCalc(player) + speed * player.mo.scale
				else
					slidespeed = (30 + speed) * player.mo.scale
				end
				//this is done to nerf the slide force underwater
				if player.mo.eflags & MFE_UNDERWATER or player.mo.eflags & 
						MFE_GOOWATER or P_InSpaceSector(player.mo)
					slidespeed = $ - ($ / 3)
				end
				P_InstaThrust(player.mo, player.drawangle, slidespeed)
				S_StartSound(player.mo, sfx_cshsld)
				crashban.slidetimer = timer
			end
			crashban.actionpress = true
		elseif player.mo.state != S_PLAY_GLIDE and crashban.slidetimer <= 0 and
				crashban.faceplant <= 0
			player.normalspeed = crawlspeed
			if player.mo.momx == player.cmomx and player.mo.momy == 
					player.cmomy
				StateSetter(player.mo, S_PLAY_CLING)
			else
				StateSetter(player.mo, S_PLAY_CLIMB)
			end
		end
	else
		if player.mo.state == S_PLAY_CLING
			player.mo.state = S_PLAY_STND
		elseif player.mo.state == S_PLAY_CLIMB
			player.mo.state = S_PLAY_WALK
		end
	end
	//this handles what happens during the slide and after
	if crashban.slidetimer > 0
		player.pflags = $ | PF_STASIS
		StateSetter(player.mo, S_PLAY_GLIDE, STR_ANIM|STR_WALL|STR_PUNCH)
		P_SpawnSkidDust(player, 10 * player.mo.scale)
		player.normalspeed = crawlspeed
		if not CootOnGround(player.mo) and IsFalling(player.mo)
			player.mo.momz = 0
		end
		crashban.slidetimer = $ - 1
	elseif player.mo.state == S_PLAY_GLIDE
		player.mo.state = S_PLAY_WALK
	end
end