local CrashSkinCheck = CrashAddon.CrashSkinCheck
local JumpManager = CrashAddon.JumpManager
local FailVibeCheck = CrashAddon.FailVibeCheck
local CootOnGround = CrashAddon.CootOnGround
local IsFalling = CrashAddon.IsFalling
local DamageRadius = CrashAddon.DamageRadius
local StateSetter = CrashAddon.StateSetter
local CanSupper = CrashAddon.CanSupper
local SpeedCalc = CrashAddon.SpeedCalc

/*has its own version of the ActionButton function since the spin move is on
different button from crawl/slide and body slam*/
local function ActionButton(player)
	if player.crash_consolemode
		return BT_SPIN
	else
		return BT_CUSTOM1
	end
end

//local variable to determine how many max button presses are allowed for glide
local max_glide_press = 8

CrashAddon.TornadoSpinGlide = function(player)
	if not CrashSkinCheck(player)
		return
	end
	
	local crashban = player.crashban
	//this resets everything and prevents moves if crash fails the vibe check
	if FailVibeCheck(player) or crashban.ropedrop or (CanSupper(player)
			and player.crash_consolemode)
		crashban.spincooldown = 0
		crashban.spinpress = false
		crashban.glidecounter = 0
		crashban.spintimer = 0
		crashban.spinslide = 0
		return
	end
	
	if crashban.spincooldown <= 0
		//this makes crash start spinning when the button is pressed
		if player.cmd.buttons & ActionButton(player) and not 
				crashban.bodyslam and not crashban.actionpress and 
				not (player.pflags & PF_SPINNING) and crashban.faceplant <= 0
			if not crashban.spinpress
				if crashban.glidecounter < max_glide_press or 
						player.powers[pw_super]
					crashban.spintimer = TICRATE / 2
					S_StartSound(player.mo, sfx_cshspn)
					if CootOnGround(player.mo)
						crashban.glidecounter = max_glide_press
					else
						crashban.glidecounter = $ + 1
					end
					//an attempt at the spin slide technique
					if crashban.slidetimer > 0 and CootOnGround(player.mo)
						crashban.spinslide = SpeedCalc(player) + 7 * 
								player.mo.scale
						crashban.slidetimer = 0
					end
				end
				crashban.spinpress = true
			end
		else
			crashban.spinpress = false
		end
		//this makes crash do damage and glide while the spin move is active
		if crashban.spintimer > 0
			StateSetter(player.mo, S_PLAY_SPINDASH, STR_ANIM|STR_ATTACK|
					STR_WALL|STR_SPIKE)
			if not CootOnGround(player.mo) and IsFalling(player.mo) and
					player.mo.momz != 0
				JumpManager(player.mo, -3 * FRACUNIT)
			end
			DamageRadius(player.mo, player.mo.radius * 2)
			if crashban.spinslide > 0
				P_InstaThrust(player.mo, player.drawangle, crashban.spinslide)
			end
			crashban.spintimer = $ - 1
		else
			if player.mo.state == S_PLAY_SPINDASH
				if CootOnGround(player.mo)
					player.mo.state = S_PLAY_WALK
				else
					player.mo.state = S_PLAY_FALL
				end
				crashban.spincooldown = TICRATE / 2
			end
			if CootOnGround(player.mo)
				crashban.glidecounter = 0
			elseif crashban.glidecounter > 0
				crashban.glidecounter = max_glide_press
			end
			crashban.spinslide = 0
		end
	else
		if player.mo.state == S_PLAY_SPINDASH
			if CootOnGround(player.mo)
				player.mo.state = S_PLAY_WALK
			else
				player.mo.state = S_PLAY_FALL
			end
		end
		crashban.glidecounter = 0
		crashban.spinslide = 0
		crashban.spincooldown = $ - 1
	end
end